/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file CStratIsendUp.h
 *        Non-blocking send based implementation for the I_CommStrategyUp interface.
 *
 * Provides the interface functionality with a simple implementation,
 * which uses non-blocking sends.
 *
 * @author Tobias Hilbrich
 * @date 03.08.2009
 */

#include "GtiDefines.h"
#include "GtiEnums.h"
#include "GtiTypes.h"
#include "I_CommStrategyUp.h"
#include "I_CommProtocol.h"
#include "ModuleBase.h"
#include "OtfTracer.h"
#include "CStratIsend.h"
#include "CStratQueue.h"

#ifndef CSTRAT_ISEND_UP_H
#define CSTRAT_ISEND_UP_H

namespace gti
{
    /**
     * Class that describes the communication protocol interface.
     */
    class CStratIsendUp : public ModuleBase<CStratIsendUp, CStratUpQueue>, public CStratIsend, public OTF_Tracer
    {
    protected:
        /** The communication protocol. */
        I_CommProtocol *protocol;

        /** The fallback communication protocol. */
        I_CommProtocol *fallback;

        unsigned int myRequest;
        unsigned long myBuf[2];
        bool myGotPing;

    public:

        /**
         * Constructor.
         * @ref ModConf see for details on module configuration.
         * @param instanceName name of the module instance.
         */
    	CStratIsendUp (const char* instanceName);

        /**
         * Destructor.
         */
        ~CStratIsendUp (void);

        /**
         * @see I_Module::isTraceable
         */
        bool isTraceable (void);

        /**
         * @see I_Traceable::setupTracing
         */
        GTI_RETURN setupTracing (GTI_TRACE_INIT_TYPE init, void *pSetupData);

        /**
         * @see I_CommStrategyUp::shutdown
         */
        GTI_RETURN shutdown (
                GTI_FLUSH_TYPE flush_behavior,
                GTI_SYNC_TYPE sync_behavior
                );

        /**
         * @see I_CommStrategyUp::send
         */
        GTI_RETURN send (
                void* buf,
                unsigned long num_bytes,
                void* free_data,
                GTI_RETURN (*buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
                );

        /**
         * @see I_CommStrategyUp::test
         */
        GTI_RETURN test (
                int* out_flag,
                unsigned long* out_num_bytes,
                void** out_buf,
                void** out_free_data,
                GTI_RETURN (**out_buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
                );

        /**
         * @see I_CommStrategyUp::wait
         */
        GTI_RETURN wait (
                unsigned long* out_num_bytes,
                void** out_buf,
                void** out_free_data,
                GTI_RETURN (**out_buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
                );

        /**
         * @see I_CommStrategyUp::flush
         */
        GTI_RETURN flush (
                void
                );

        /**
         * @see I_CommStrategyUp::enforce_blocking_comm
         */
        GTI_RETURN enforce_blocking_comm (
                void
                );

        /**
         * @see I_CommStrategyUp::activate_fallback_comm
         */
        GTI_RETURN activate_fallback_comm (
                void
                );
    };
}

#endif /* CSTRAT_ISEND_UP_H */
