/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file CStratSimpleDown.h
 *       Implementation for the I_CommStrategyDown interface.
 *
 * Sends all messages immediately and blocking.
 *
 * @author Tobias Hilbrich
 * @date 15.04.2009
 */

#include "GtiDefines.h"
#include "GtiEnums.h"
#include "GtiTypes.h"
#include "I_CommStrategyDown.h"
#include "I_CommProtocol.h"
#include "ModuleBase.h"
#include "OtfTracer.h"
#include "CStratQueue.h"

#ifndef CSTRAT_SIMPLE_DOWN_H
#define CSTRAT_SIMPLE_DOWN_H

namespace gti
{
    /**
     * Simple implementation of upwards strategy.
     * Properties:
     * - no aggregation
     * - uses ssend to send (upwards and downwards)
     * - sends an acknowledgment after a received message was processed
     */
    class CStratSimpleDown : public ModuleBase<CStratSimpleDown, CStratDownQueue>, public OTF_Tracer
    {
    protected:
        /** The communication protocol. */
        I_CommProtocol *protocol;

        /** The fallback communication protocol. */
        I_CommProtocol *fallback;

        unsigned int  test_request;
        unsigned long test_buff[2];
        static const unsigned long myTokenShutdownSync;
        static const unsigned long myTokenMessage;
        static const unsigned long myTokenAck;

    public:

        /**
         * Constructor.
         * @ref ModConf see for details on module configuration.
         * @param instanceName name of the module instance.
         */
        CStratSimpleDown (const char* instanceName);

        /**
         *Destructor.
         */
        ~CStratSimpleDown (void);

        /**
         * @see I_Module::isTraceable
         */
        bool isTraceable (void);

        /**
         * @see I_Traceable::setupTracing
         */
        GTI_RETURN setupTracing (GTI_TRACE_INIT_TYPE init, void *pSetupData);

        /**
         * @see I_CommStrategyDown::getNumClients
         */
        GTI_RETURN getNumClients (unsigned long *outNumClients);

        /**
         * @see I_CommStrategyDown::shutdown
         */
        GTI_RETURN shutdown (
                GTI_FLUSH_TYPE flush_behavior,
                GTI_SYNC_TYPE sync_behavior);

        /**
         * @see I_CommStrategyDown::broadcast
         */
        GTI_RETURN broadcast (
                void* buf,
                unsigned long num_bytes,
                void* free_data,
                GTI_RETURN (*buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
                );

        /**
         * @see I_CommStrategyDown::test
         */
        GTI_RETURN test (
                int* out_flag,
                unsigned long* out_num_bytes,
                void** out_buf,
                void** out_free_data,
                GTI_RETURN (**out_buf_free_function) (void* free_data, unsigned long num_bytes, void* buf),
                unsigned long *outChannel
                );

        /**
         * @see I_CommStrategyDown::acknowledge
         */
        GTI_RETURN acknowledge (
        		unsigned long channel
        );

        /**
         * @see I_CommStrategyDown::wait
         */
        GTI_RETURN wait (
                unsigned long* out_num_bytes,
                void** out_buf,
                void** out_free_data,
                GTI_RETURN (**out_buf_fee_function) (void* free_data, unsigned long num_bytes, void* buf),
                unsigned long *outChannel
                );

        /**
         * @see I_CommStrategyDown::flush
         */
        GTI_RETURN flush (
                void
                );

        /**
         * @see I_CommStrategyDown::enforce_blocking_comm
         */
        GTI_RETURN enforce_blocking_comm (
                void
                );

        /**
         * @see I_CommStrategyDown::activate_fallback_comm
         */
        GTI_RETURN activate_fallback_comm (
                void
                );
    };
}

#endif /* CSTRAT_SIMPLE_DOWN_H */
