/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file GtiTraceDefines.h
 * This header defines ids and names for tracing.
 * All modules that support tracing should define their
 * function ids etc. in this header file.
 * Ids should be unique to avoid a need for unification.
 *
 * @author Tobias Hilbrich
 * @date 04-14-2008
 */

//Function group identifiers
#define FUNCTION_GROUP_ID__COMMUNICATE              1
#define FUNCTION_GROUP_NAME__COMMUNICATE            "GTI-Comm"
#define FUNCTION_GROUP_ID__IDLE                     2
#define FUNCTION_GROUP_NAME__IDLE                   "GTI-Idle"
#define FUNCTION_GROUP_ID__PROCESSING               3
#define FUNCTION_GROUP_NAME__PROCESSING             "GTI-Processing"
#define FUNCTION_GROUP_ID__APP_MPI                  4
#define FUNCTION_GROUP_NAME__APP_MPI                "App-MPI"
#define FUNCTION_GROUP_ID__APP_PROCESSING           5
#define FUNCTION_GROUP_NAME__APP_PROCESSING         "App-Processing"

//Function identifiers
#define THREADED_MPI_PLACE_FUNC__main               1
#define THREADED_MPI_PLACE_FUNC__setupTracing       2
#define THREADED_MPI_PLACE_FUNC__Destructor         3
#define THREADED_MPI_PLACE_FUNC__run                4

#define COMM_STRATEGY_DOWN__setupTracing            100
#define COMM_STRATEGY_DOWN__Destructor              101
#define I_COMM_STRATEGY_DOWN__shutdown              102
#define I_COMM_STRATEGY_DOWN__getNumClients         103
#define I_COMM_STRATEGY_DOWN__broadcast             104
#define I_COMM_STRATEGY_DOWN__test                  105
#define I_COMM_STRATEGY_DOWN__wait                  106
#define I_COMM_STRATEGY_DOWN__flush                 107
#define I_COMM_STRATEGY_DOWN__enforce_blocking_comm 108
#define I_COMM_STRATEGY_DOWN__activate_fallback_comm 109

#define COMM_STRATEGY_UP__setupTracing              200
#define COMM_STRATEGY_UP__Destructor                201
#define I_COMM_STRATEGY_UP__shutdown                202
#define I_COMM_STRATEGY_UP__send                    203
#define I_COMM_STRATEGY_UP__test                    204
#define I_COMM_STRATEGY_UP__wait                    205
#define I_COMM_STRATEGY_UP__flush                   206
#define I_COMM_STRATEGY_UP__enforce_blocking_comm   207
#define I_COMM_STRATEGY_UP__activate_fallback_comm  208

#define COMM_PROTOCOL__setupTracing                 300
#define COMM_PROTOCOL__Destructor                   301
#define I_COMM_PROTOCOL__shutdown                   302
#define I_COMM_PROTOCOL__getNumChannels             303
#define I_COMM_PROTOCOL__ssend                      304
#define I_COMM_PROTOCOL__isend                      305
#define I_COMM_PROTOCOL__recv                       306
#define I_COMM_PROTOCOL__irecv                      307
#define I_COMM_PROTOCOL__test_msg                   308
#define I_COMM_PROTOCOL__wait_msg                   309
#define I_COMM_PROTOCOL__removeOutstandingRequests  310
#define COMM_PROTOCOL__startup                      311

#define PLACE_RECEIVAL__setupTracing                400
#define PLACE_RECEIVAL__Destructor                  401
#define I_PLACE_RECEIVAL__ReceiveRecord             402

#define LOGGER__get_msg                             500

/*CHECKS*/
#define CHECK__comm_valid                           1000
#define CHECK__request_valid                        1001
#define CHECK__tag_valid                            1002

/*STATE TRACKERS*/
#define STATE_TRACKER_COMM__add                     2000
#define STATE_TRACKER_COMM__remove                  2001
#define STATE_TRACKER_COMM__exists                  2002

#define STATE_TRACKER_REQUEST__add                  2100
#define STATE_TRACKER_REQUEST__remove               2101
#define STATE_TRACKER_REQUEST__exists               2102

/*WRAPPER*/
#define WRAPPER__main                               3000
#define WRAPPER__APP_PROCESSING                     3997
#define WRAPPER__setupTracing                       3999
#define WRAPPER__Destructor                         3998
#define WRAPPER__MPI_Abort        3001
#define WRAPPER__MPI_Address        3002
#define WRAPPER__MPI_Allgather        3003
#define WRAPPER__MPI_Allgatherv        3004
#define WRAPPER__MPI_Allreduce        3005
#define WRAPPER__MPI_Alltoall        3006
#define WRAPPER__MPI_Alltoallv        3007
#define WRAPPER__MPI_Attr_delete        3008
#define WRAPPER__MPI_Attr_get        3009
#define WRAPPER__MPI_Attr_put        3010
#define WRAPPER__MPI_Barrier        3011
#define WRAPPER__MPI_Bcast        3012
#define WRAPPER__MPI_Bsend        3013
#define WRAPPER__MPI_Bsend_init        3014
#define WRAPPER__MPI_Buffer_attach        3015
#define WRAPPER__MPI_Buffer_detach        3016
#define WRAPPER__MPI_Cancel        3017
#define WRAPPER__MPI_Cart_coords        3018
#define WRAPPER__MPI_Cart_create        3019
#define WRAPPER__MPI_Cart_get        3020
#define WRAPPER__MPI_Cart_map        3021
#define WRAPPER__MPI_Cart_rank        3022
#define WRAPPER__MPI_Cart_shift        3023
#define WRAPPER__MPI_Cart_sub        3024
#define WRAPPER__MPI_Cartdim_get        3025
#define WRAPPER__MPI_Comm_compare        3026
#define WRAPPER__MPI_Comm_create        3027
#define WRAPPER__MPI_Comm_dup        3028
#define WRAPPER__MPI_Comm_free        3029
#define WRAPPER__MPI_Comm_group        3030
#define WRAPPER__MPI_Comm_rank        3031
#define WRAPPER__MPI_Comm_remote_group        3032
#define WRAPPER__MPI_Comm_remote_size        3033
#define WRAPPER__MPI_Comm_size        3034
#define WRAPPER__MPI_Comm_split        3035
#define WRAPPER__MPI_Comm_test_inter        3036
#define WRAPPER__MPI_Dims_create        3037
#define WRAPPER__MPI_Errhandler_create        3038
#define WRAPPER__MPI_Errhandler_free        3039
#define WRAPPER__MPI_Errhandler_get        3040
#define WRAPPER__MPI_Errhandler_set        3041
#define WRAPPER__MPI_Error_class        3042
#define WRAPPER__MPI_Error_string        3043
#define WRAPPER__MPI_Finalize        3044
#define WRAPPER__MPI_Gather        3045
#define WRAPPER__MPI_Gatherv        3046
#define WRAPPER__MPI_Get_count        3047
#define WRAPPER__MPI_Get_elements        3048
#define WRAPPER__MPI_Get_processor_name        3049
#define WRAPPER__MPI_Get_version        3050
#define WRAPPER__MPI_Graph_create        3051
#define WRAPPER__MPI_Graph_get        3052
#define WRAPPER__MPI_Graph_map        3053
#define WRAPPER__MPI_Graph_neighbors        3054
#define WRAPPER__MPI_Graph_neighbors_count        3055
#define WRAPPER__MPI_Graphdims_get        3056
#define WRAPPER__MPI_Group_compare        3057
#define WRAPPER__MPI_Group_difference        3058
#define WRAPPER__MPI_Group_excl        3059
#define WRAPPER__MPI_Group_free        3060
#define WRAPPER__MPI_Group_incl        3061
#define WRAPPER__MPI_Group_intersection        3062
#define WRAPPER__MPI_Group_rank        3063
#define WRAPPER__MPI_Group_size        3064
#define WRAPPER__MPI_Group_translate_ranks        3065
#define WRAPPER__MPI_Group_union        3066
#define WRAPPER__MPI_Ibsend        3067
#define WRAPPER__MPI_Init        3068
#define WRAPPER__MPI_Init_thread        3069
#define WRAPPER__MPI_Initialized        3070
#define WRAPPER__MPI_Intercomm_create        3071
#define WRAPPER__MPI_Intercomm_merge        3072
#define WRAPPER__MPI_Iprobe        3073
#define WRAPPER__MPI_Irecv        3074
#define WRAPPER__MPI_Irsend        3075
#define WRAPPER__MPI_Isend        3076
#define WRAPPER__MPI_Issend        3077
#define WRAPPER__MPI_Keyval_create        3078
#define WRAPPER__MPI_Keyval_free        3079
#define WRAPPER__MPI_Op_create        3080
#define WRAPPER__MPI_Op_free        3081
#define WRAPPER__MPI_Pack        3082
#define WRAPPER__MPI_Pack_size        3083
#define WRAPPER__MPI_Probe        3084
#define WRAPPER__MPI_Recv        3085
#define WRAPPER__MPI_Recv_init        3086
#define WRAPPER__MPI_Reduce        3087
#define WRAPPER__MPI_Reduce_scatter        3088
#define WRAPPER__MPI_Request_free        3089
#define WRAPPER__MPI_Rsend        3090
#define WRAPPER__MPI_Rsend_init        3091
#define WRAPPER__MPI_Scan        3092
#define WRAPPER__MPI_Scatter        3093
#define WRAPPER__MPI_Scatterv        3094
#define WRAPPER__MPI_Send        3095
#define WRAPPER__MPI_Send_init        3096
#define WRAPPER__MPI_Sendrecv        3097
#define WRAPPER__MPI_Sendrecv_replace        3098
#define WRAPPER__MPI_Ssend        3099
#define WRAPPER__MPI_Ssend_init        3100
#define WRAPPER__MPI_Start        3101
#define WRAPPER__MPI_Startall        3102
#define WRAPPER__MPI_Test        3103
#define WRAPPER__MPI_Test_cancelled        3104
#define WRAPPER__MPI_Testall        3105
#define WRAPPER__MPI_Testany        3106
#define WRAPPER__MPI_Testsome        3107
#define WRAPPER__MPI_Topo_test        3108
#define WRAPPER__MPI_Type_commit        3109
#define WRAPPER__MPI_Type_contiguous        3110
#define WRAPPER__MPI_Type_extent        3111
#define WRAPPER__MPI_Type_free        3112
#define WRAPPER__MPI_Type_hindexed        3113
#define WRAPPER__MPI_Type_hvector        3114
#define WRAPPER__MPI_Type_indexed        3115
#define WRAPPER__MPI_Type_lb        3116
#define WRAPPER__MPI_Type_size        3117
#define WRAPPER__MPI_Type_struct        3118
#define WRAPPER__MPI_Type_ub        3119
#define WRAPPER__MPI_Type_vector        3120
#define WRAPPER__MPI_Unpack        3121
#define WRAPPER__MPI_Wait        3122
#define WRAPPER__MPI_Waitall        3123
#define WRAPPER__MPI_Waitany        3124
#define WRAPPER__MPI_Waitsome        3125
