/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file I_Module.h
 *       Base interface supported by all modules.
 *
 * @author Tobias Hilbrich
 */

#include "GtiDefines.h"
#include "GtiEnums.h"
#include "GtiTypes.h"

#include <string>

#ifndef I_MODULE_H
#define I_MODULE_H

namespace gti
{
    /**
     * Interface used by all modules.
     */
    class I_Module
    {
    public:

    	/**
    	 * Virtual destructor.
    	 */
    	virtual ~I_Module () {}

        /**
         * Returns the name of this module.
         * @return name of this module.
         */
        virtual std::string getName (void) = 0;

        /**
         * Returns whether this module supports
         * tracing or not.
         * @return true if traceable, flase otherwise.
         */
        virtual bool isTraceable (void) = 0;

        /**
         * Sets up the tracing implementation.
         * This function may only be called for modules that return
         * true for isTraceable.
         * @param init type of tracing setup (root, copy, child).
         * @param pSetupData data needed for the specific setup.
         * @return GTI_SUCCESS if successful.
         */
        virtual GTI_RETURN setupTracing (GTI_TRACE_INIT_TYPE init, void *pSetupData) = 0;
    }; /*class I_Module*/
} /*namespace gti*/

#endif /* I_MODULE_H */
