/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file ThreadedMpiPlace.h
 *       Header for a threaded MPI application based
 *       placement driver.
 *
 * @author Tobias Hilbrich
 *
 */

#include "GtiDefines.h"
#include "GtiEnums.h"
#include "GtiTypes.h"
#include "I_Module.h"
#include "ModuleBase.h"
#include "OtfTracer.h"
#include "I_CommStrategyDown.h"
#include "I_PlaceReceival.h"

#ifndef THREADED_MPI_PLACE_H
#define THREADED_MPI_PLACE_H

namespace gti
{
    /**
     * Class for a multi-threaded MPI based placement
     * driver.
     *
     * Needs at least 2 sub modules:
     *  1st: Comm Strategy downwards (to receive trace information)
     *  2nd: Receival module (to process and forward trace information
     *  [Optional] 3rd: Wrapper module runing on this place
     *                  Use case is to shut down the wrapper when the place
     *                  finishes.
     */
    class ThreadedMPIPlace : public ModuleBase<ThreadedMPIPlace, I_Module>, public OTF_Tracer
    {
    protected:
        I_CommStrategyDown* myTraceRecv;
        I_PlaceReceival*    myReceival;
        I_Module*			myWrapper;

        //TODO
        //I_MgmtHandler*    myMgmtHandler;


    public:
        /**
         * Constructor.
         * Sets up the this tool place with all its modules.
         * @param instanceName name of this module instance
         */
        ThreadedMPIPlace (const char* instanceName);

        /**
         * Destructor.
         */
        ~ThreadedMPIPlace (void);

        /**
         * @see I_Module::isTraceable
         */
        bool isTraceable (void);

        /**
         * @see I_Traceable::setupTracing
         */
        GTI_RETURN setupTracing (GTI_TRACE_INIT_TYPE init, void *pSetupData);

        /**
         * Starts the tool place.
         */
        void run (void);

    }; /*class ThreadedMPIPlace*/
} /*namespace gti*/

#endif /* THREADED_MPI_PLACE_H */
