/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2010-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2010-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file ArgumentInput.cpp
 * 		@see gti::weaver::ArgumentInput
 *
 * @author Tobias Hilbrich
 * @date 12.01.2010
 */

#include "ArgumentInput.h"

using namespace gti::weaver::calls;

//=============================
// ArgumentInput
//=============================
ArgumentInput::ArgumentInput ( )
 : Input (),
   myTargetArgument (NULL)
{

}

//=============================
// ArgumentInput
//=============================
ArgumentInput::ArgumentInput (Argument* argument)
 : Input (),
   myTargetArgument (argument)
{

}

//=============================
// ArgumentInput
//=============================
ArgumentInput::~ArgumentInput ( )
{
	myTargetArgument = NULL;
}

//=============================
// setTargetArgument
//=============================
void ArgumentInput::setTargetArgument ( Argument * new_var )
{
  myTargetArgument = new_var;
}

//=============================
// getMyTargetArgument
//============================
Argument * ArgumentInput::getTargetArgument ( )
{
  return myTargetArgument;
}

//=============================
// getName
//=============================
std::string ArgumentInput::getName ( ) const
{
	return myTargetArgument->getName();
}

//=============================
// getType
//=============================
std::string ArgumentInput::getType ( ) const
{
	return myTargetArgument->getType();
}

//=============================
// isArrayInput
//=============================
bool ArgumentInput::isArrayInput () const
{
	if (myTargetArgument->isArray())
		return true;
	return false;
}

//=============================
// getLenName
//=============================
std::string ArgumentInput::getLenName () const
{
	if (isArrayInput())
		return myTargetArgument->getLengthVariable ();

	return "";
}

//=============================
// print
//=============================
std::ostream& ArgumentInput::print (std::ostream& out) const
{
	out << "argumentInput{";
	Input::print(out);
	out << "}";

	return out;
}

//=============================
// getDotInputNodeName
//=============================
std::string ArgumentInput::getDotInputNodeName (std::string callName)
{
	return callName + ":" + myTargetArgument->getName();
}

/*EOF*/
