/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2010-2011
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2010-2011
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file Adjacency.cpp
 * 		@see gti::weaver::Adjacency
 *
 * @author Tobias Hilbrich
 * @date 13.01.2010
 */

#include "Adjacency.h"

using namespace gti::weaver::layout;

//=============================
// Adjacency
//=============================
Adjacency::Adjacency (Level* target, Communication* comm)
:  target (target),
   comm (comm)
{

}

//=============================
// Adjacency
//=============================
Adjacency::Adjacency ()
: target (NULL),
  comm (NULL)
{

}

//=============================
// ~Adjacency
//=============================
Adjacency::~Adjacency ()
{
	/*Do not free the pointers,
	 *their storage is managed by
	 *their respective singletons.
	 */
	target = NULL;
	comm = NULL;
}

//=============================
// getTarget
//=============================
Level* Adjacency::getTarget (void)
{
	return target;
}

//=============================
// getComm
//=============================
Communication* Adjacency::getComm (void)
{
	return comm;
}

//=============================
// print
//=============================
std::ostream& Adjacency::print (std::ostream& out) const
{
	if (!target || !comm)
	{
		out << "invalidAdjacency";
		return out;
	}

	out
		<< "adjacency={targetLevel="
		<< target->getOrder()
		<< ", "
		<< *comm
		<< "}";

	return out;
}

/*EOF*/
